# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"extract_ukpn_single_incident_ids": "01-retrieval.ipynb",
         "extract_ukpn_multiple_incident_urls": "01-retrieval.ipynb",
         "extract_ukpn_multiple_incident_ids": "01-retrieval.ipynb",
         "get_ukpn_incident_detail_url": "01-retrieval.ipynb",
         "is_ukpn_incident_active": "01-retrieval.ipynb",
         "extract_ukpn_relevant_info": "01-retrieval.ipynb",
         "get_ukpn_incidents_info": "01-retrieval.ipynb",
         "save_json_data": "01-retrieval.ipynb",
         "clean_ssen_incident_info": "01-retrieval.ipynb",
         "get_ssen_incidents_info": "01-retrieval.ipynb",
         "get_wpd_incident_feed": "01-retrieval.ipynb",
         "wpd_incident_id_to_url": "01-retrieval.ipynb",
         "clean_wpd_incident_info": "01-retrieval.ipynb",
         "get_wpd_incidents_info": "01-retrieval.ipynb",
         "get_raw_sp_incidents_info": "01-retrieval.ipynb",
         "clean_sp_incident_info": "01-retrieval.ipynb",
         "get_sp_incidents_info": "01-retrieval.ipynb",
         "get_np_auth": "01-retrieval.ipynb",
         "get_np_r": "01-retrieval.ipynb",
         "get_np_raw_incidents_info": "01-retrieval.ipynb",
         "clean_np_incident_info": "01-retrieval.ipynb",
         "get_np_incidents_info": "01-retrieval.ipynb",
         "get_enw_incidents_page": "01-retrieval.ipynb",
         "check_num_results": "01-retrieval.ipynb",
         "get_enw_raw_incidents": "01-retrieval.ipynb",
         "clean_enw_incident_info": "01-retrieval.ipynb",
         "get_enw_incidents_info": "01-retrieval.ipynb",
         "collate_cleaned_incidents_info": "02-collation.ipynb",
         "get_combined_outages_df": "02-collation.ipynb",
         "check_postcode_is_in_df": "02-collation.ipynb",
         "get_postcode_location_df": "02-collation.ipynb",
         "get_postcode_url": "02-collation.ipynb",
         "get_postcode_location_api": "02-collation.ipynb",
         "PostCodeLocator": "02-collation.ipynb",
         "add_outage_longitudes_latitudes": "02-collation.ipynb",
         "construct_outages_gdf": "02-collation.ipynb",
         "generate_map_js": "02-collation.ipynb",
         "clean_outages_df": "02-collation.ipynb",
         "generate_index_md": "02-collation.ipynb",
         "app": "03-deployment.ipynb",
         "update_docs": "03-deployment.ipynb"}

modules = ["retrieval.py",
           "collation.py",
           "deployment.py"]

doc_url = "https://AyrtonB.github.io/outage/"

git_url = "https://github.com/AyrtonB/Outage-Watch"

def custom_doc_links(name): return None
